;============================================================
;= Copyright  2001 Tayfun Gms                            =
;= Source code is free for use with MAMP-OS smart card only =
;= All other usage is prohibited                            =
;= T=E Protocol example user source code for MAMP-OS V2.0   =
;= Set tabulator size to 9                                  =
;============================================================

.include	"v20_64c.inc"
.nolist

;------------------------------------------------------------------------------

.equ	ATRLEN	=	17		;ATR length
.equ	PINLEN	=	4		;8 digit PIN
.equ	CLKBAUD	=	_baud_19200_3579

;------------------------------------------------------------------------------

.cseg
.org 0					;mandatory

.include	"init.asm"			;set protocol parameters
.include	"atr.asm"			;send out answer to reset

;------------------------------------------------------------------------------

start:	ldi	yl,low(RAM_buf + $100)	;clear user RAM
	ldi	yh,high(RAM_buf + $100)
	clr	r16
_crm1:	st	-y,r16
	tst	yl
	brne	_crm1

	ldi	r17,_sc_rx_T_E		;receive request in T=E protocol
	call	ADR_system_call

	breq	_rok0			;Z flag set if correct checksum 
	rjmp	_err5			;checksum verification failed

_rok0:	ldi	zl,low(RAM_buf)
	ldi	zh,high(RAM_buf)

	ldd	r16,z+1			;get CLA
	cpi	r16,$B2
	breq	c_B2_
	cpi	r16,$B3
	breq	c_B3_
	rjmp	_err1			;undefined class

c_B2_:	rjmp	CLA_B2			;crypto related commands
c_B3_:	rjmp	CLA_B3			;PIN related commands

;------------------------------------------------------------------------------

CLA_B2:	ldd	r16,z+2			;get INS
	cpi	r16,$00
	breq	i_B2_00
	cpi	r16,$10
	breq	i_B2_10
	cpi	r16,$20
	breq	i_B2_20
	rjmp	_err2			;undefined instruction

i_B2_00:	rjmp	_i_B2_00			;RSA
i_B2_10:	rjmp	_i_B2_10			;AES
i_B2_20:	rjmp	_i_B2_20			;random number

_i_B2_00:
.include	"CLA_B2\INS_00.asm"
_i_B2_10:
.include	"CLA_B2\INS_10.asm"
_i_B2_20:
.include	"CLA_B2\INS_20.asm"

;------------------------------------------------------------------------------

CLA_B3:	ldd	r16,z+2			;get INS
	cpi	r16,$1E
	breq	i_B3_1E
	cpi	r16,$AA
	breq	i_B3_AA
	rjmp	_err2			;undefined instruction

i_B3_1E:	rjmp	_i_B3_1E			;change PIN
i_B3_AA:	rjmp	_i_B3_AA			;enter user code upload mode upon valid PIN

_i_B3_1E:
.include	"CLA_B3\INS_1E.asm"
_i_B3_AA:
.include	"CLA_B3\INS_AA.asm"

;------------------------------------------------------------------------------

_err0:	clr	r16			;LEN = 0
_suc0:	ldi	zl,low(RAM_buf)
	ldi	zh,high(RAM_buf)
	std	z+7,r16			;LEN

	ldi	r17,_sc_rd_RC_high	;get return code high byte in r16
	call	ADR_system_call
	std	z+5,r16

	ldi	r17,_sc_rd_RC_low		;get return code low byte in r16
	call	ADR_system_call
	std	z+6,r16

	ldi	r17,_sc_tx_T_E		;transmit answer in T=E protocol
	call	ADR_system_call

;------------------------------------------------------------------------------

	rjmp	start

;------------------------------------------------------------------------------

.include	"rc.asm"				;return codes
.include	"eeprom.asm"			;user EEPROM

.exit
